;
;   dsproc.asm
;
;   john conner wd0fhg
;   may 1988
;
;   This routine for the TMS32010 is intended as a sample for processing
;   routines to be run from the dspscope program.  This routine pickes up data
;   from the PC, sends it to the processing routine, and then sends output
;   data back to the PC.  The 8-bit register is used to tell the PC when
;   processing is done.  The routine sets in a wait loop until interrupted
;   by the PC, indicating that new data is available to process. The return
;   address for the interrupt is ignored and the entry to and return from
;   the processing routine are simple branches, thus the stack is empty
;   on entry to the processing routine.
;   Although the code is placed in low memory only the two jumps at 0x0 and
;   0x2 need to be anywhere special.
;
;   In order to properly interface with DSPSCOPE the conversions to and from
;   Q15 format shown in the proc: section should be used.  The conversion
;   in the Dalanco book on page 14.1 do not produce the correct offset
;   binary output ( they do seem to work with the D/A since it only sees
;   12 bits anyway ).
;
;   **************************** NOTE ******************************
;   This routine makes use of the INT input from the PC. Therefore, jumper
;   J4 must be placed in the "PC" position.
;

data:	equ 0	;the data input from the PC before processing and the
		;processed data to be output to the PC
one:	equ 1	;storage for constant 1
cntr:	equ 2	;storage for counter
temp:	equ 3	;temp storage
mask:	equ 4	;data mask to conversion

	org	0h
	b	start	; on reset we go to start
	b	getit	; on an interrupt from the PC we go read data

	org	00ah
cnsts:	dw	0f7ffh	; conversion mask

start:	ldpk	0	; set pointer to data page 0
	lack	cnsts	; move mask to data mem
	tblr	mask
	lack	1	; setup data counter (just tells PC we're processing)
	sacl	cntr
	lack	1	; put constant in data memory
	sacl	one
	out	cntr, pa5   ; send current count to PC

	eint		; enable interrupts from the PC

wait:	b	wait	; sleep until we get an interrupt from PC

getit:	in	data, pa7   ; get the next data point from the PC
	b	proc	    ; do processing
done:	out	data, pa7   ; send results to PC
	lac	cntr	    ; update counter so PC knows
	add	one	    ; we have received and processed the data
	sacl	cntr
	out	cntr, pa5   ; send current count to PC
	eint		    ; enable next interrupt
	b	wait	    ; ignor the ret on stack -- we go back to wait
			    ; and start over.....

proc:	nop		    ; put the processing routine here
	nop		    ; for now data out = data in
	lac	data,4	    ; convert to two complement Q15
	sub	one,15
	sacl	data
;
	lac	data,12     ; convert back to offset binary
	subh	mask
	sach	data
;
	b	done	    ; be careful using subroutines -- stack is only
			    ; 4 deep unless you build your own
			    ; use the " b proc" and "b done" to save stack
			    ; space
