/***************************************************************************/
/*			  DSPSCOPE  Ver 0.10				   */
/*									   */
/*     General purpose oscilloscope for use with the Delanco Spry	   */
/*     Model 10 Data Acquisition and signal processing board.		   */
/*     This program provides a general user interface for developing	   */
/*     signal processing algorithms for the TMD32010 on the Model 10.	   */
/*     It allows the user to record and play back data, inspect and	   */
/*     modify the data and to process data with the Model 10. Access	   */
/*     to the Delanco Spry Debugger (D320D.EXE), a text editor of the	   */
/*     users choice (subject to memory limits) and the MAC320 macro	   */
/*     assembler are supported from within the program. The oscilloscope   */
/*     graphics support color on EGA systems and B/W on CGA. An 80x87	   */
/*     coprocessor is supported if present.				   */
/*									   */
/*									   */
/*     This version (ver 0.1) supports the following functions: 	   */
/*	    o....2 chan oscilloscope type display for viewing data with    */
/*		 controls to change display scales and to pan through	   */
/*		 stored data.						   */
/*	    o....EGA color, CGA B/W, and 80x87 support is automatic.	   */
/*	    o....load data previously stored with FROM320 for review.	   */
/*	    o....access the Delanco Spry Debugger D320D.EXE.		   */
/*	    o....access the MAC320 assembler.				   */
/*	    o....access a text editor of the users choice.		   */
/*	    o....collect data directly from the Model 10 (FROM320).	   */
/*	    o....save data for later review.				   */
/*	    o....load and run a TMS320 program to process stored data	   */
/*		 and display both input and output signals.		   */
/*									   */
/*     Future versions will include the following functions:		   */
/*	    o....output data directly to the Model 10 (TO320).		   */
/*	    o....edit data in the buffer.				   */
/*	    o....compute and display fft of data in display window.	   */
/*									   */
/*			DSP Scope Commands				   */
/*		  F1	      -- gather data (FROM320)			   */
/*		  F2	      -- send data  (TO320) --- NYI		   */
/*		  F3	      -- load TMS320 program			   */
/*		  F4	      -- run D320D debugger			   */
/*		  F5	      -- start text editor			   */
/*		  F6	      -- run MAC320 assembler			   */
/*		  F7	      -- process data with TMS320 program	   */
/*		  F8	      -- toggle displayed data			   */
/*		  F9	      -- load buffer from data file		   */
/*		  F10	      -- save buffer to data file		   */
/*	  Unused function keys cause current screen to be redisplayed.	   */
/*		  Left arrow  -- move forward in data			   */
/*		  Right arrow -- move backward in data			   */
/*		  Up arrow    -- increase volts/div			   */
/*		  Down arrow  -- decrease volts/div			   */
/*		  Home	      -- increase sec/div			   */
/*		  End	      -- decrease sec/div			   */
/*		  PgUp	      -- increase data step rate		   */
/*		  PgDn	      -- decrease data step rate		   */
/*		  Esc	      -- exit from program			   */
/*		  ?	      -- display commands screen		   */
/*									   */
/*     Standard AMSAT/TAPR setup of memory at 0xd000 and io ports at	   */
/*     0x300 is assumed. The files D320D.EXE, MAC320.EXE, INTEL.EXE and    */
/*     your editor must be in a directory listed in the PATH.  The files   */
/*     GATHER.HEX, SEND.HEX, STRTHRU.HEX and any other 320 ASM or HEX	   */
/*     files you want to work with need to be in your current directory.   */
/*     To use a program on the DS320 to process data in the buffer you	   */
/*     will need to move jumper 4 to the PC position. This jumper controls */
/*     the source of interrupts to the 32010.  Since most of the code	   */
/*     so far uses BIO to control sampling, I chose to use INT to allow    */
/*     the PC to control the 32010 with out doing a restart all the time.  */
/*									   */
/*     If some of the code looks familiar it is just what is left from	   */
/*     the little change I was going to make to FROM320.C by N4HY/W3IWI.   */
/*     I just wanted to check some data I had collected.;-> In the process */
/*     I found what appears to be a bug in the MSC 5.0 compiler while	   */
/*     trying to get the collection part of this to work.  DO NOT use	   */
/*     the /Oatl option with the compiler. The /Oat seems OK but when	   */
/*     compiled with /Oatl the loop in fulbuf() gets screwed up and	   */
/*     you don't get proper sampling. In this program you get ONE sample   */
/*     in each buffer. In FROM320.C you get about everyother sample.	   */
/*     I don't know if this is related to another bug that has been        */
/*     reported that put code in the data segment or some such. 	   */
/*									   */
/*     Sometime I will break this up into smaller files!		   */
/*									   */
/*									   */
/*     Questions, comments, criticisms, gripes can be sent to:		   */
/*		John Conner, WD0FHG					   */
/*		3631 Brentwood Terrace					   */
/*		Colorado Springs, Colorado 80910			   */
/*		(719) 597-5407						   */
/*		cis id.  72165,743					   */
/*		arpanet  conner%winfree.uucp@ucsd.edu			   */
/*									   */
/*									   */
/*     Of course we will probably just ignor the criticisms and cripes;-)  */
/*									   */
/*     Copyright (C) John Conner, WD0FHG, 1988. 			   */
/*	   Permission granted for non-commerical use.			   */
/*									   */
/*     Revisions:							   */
/*     <880418>  j. conner						   */
/*		 Initial release version 0.00. (it got lost on the way	   */
/*		 to TOMCAT ).						   */
/*									   */
/*     <880506>  j. conner    version 0.10				   */
/*		 Moved video stuff to a seprate file ( dspsvid.c ).	   */
/*		 Made header file for common stuff (dspscope.h).	   */
/*		 Made display 2 channel with fixed offsets.		   */
/*		 Added routine to send data to 32010 for processing,	   */
/*		 see dsproc.asm for sample processing file.		   */
/*									   */
/*     <880506>  j. conner    version 0.11				   */
/*		 Corrected problem with scaling for CGA. Added		   */
/*		 _setbkcolor to force background to black. Added display   */
/*		 of version number. Still have problem with F6 (mac320	   */
/*		 assemble command), always finds blank string input on	   */
/*		 first try.						   */
/*									   */
/***************************************************************************/

#include <stdio.h>
#include <process.h>
#include <graph.h>

#ifndef FALSE
#define FALSE 0
#define TRUE  !FALSE
#endif

int *storage[8],k,count;
unsigned i;
unsigned prt0,cntrlpt,prt1,prt2;

char version[] = "0.11";
struct	videoconfig config;
FILE *fp;	       /* current input file */
int xmin, ymin;        /* Min x and y coord. */
int xmax, ymax;        /* Max x and y coord  */
int lxmin, lymin;      /* Logical Min x and y coord. */
int lxmax, lymax;      /* Logical Max x and y coord  */
int xstep,ystep;       /* grid steps for x and y */
int maxxres,maxyres;   /* x and y resolution in pixels (nearest power of 2 ) */
unsigned int scale ;   /* display vert scale value */
int tbase = 1;	       /* display horz scale value */
int dstep = 1;	       /* data step rate */
int ioff;	       /* current display offset in buffer */
int which;	       /* current display buffer */
int offset1;	       /* dc offset of channel 1 (input ) init +1/2 scale */
int offset2;	       /* dc offset of channel 2 (output) init -1/2 scale */
float srate;	       /* sample rate of data */
int lastbuf;	       /* number of last buffer in storage */
int dsply_chn;	       /* channel to display */

#define CONTROL 0x34
#define CONT2 0xb4
#define MAXBLK 4	  /* maximum number of blocks in file to process */
#define VOLTS  0.0024	  /* volts per count from A/D */
#define XGRIDS 10
#define YGRIDS 5
#define BOTH 2
#define CH1  0
#define CH2  1

void init_video(void);
void close_video(void);
void drawscrn(void);
FILE *getfile(void);
void edtbuf(void);
void newscreen(int ii, int which, int trace );
void drw_trc1(int ii, int which);
void drw_trc2(int ii, int which);
void showhelp(void);
void signon(void);
void data_input(void);
void data_io(int dir);
void load_320(void);
void run_debug( void );
void strtedit( void );
void run_mac320( void );
void proc_320(void);
void control_timer(long cnt, float osc, float *trate);
void fulbuf(int baseio, int which);
int ci(void);
void cflush(void);
void pktc(void);

main(argc,argv)
int argc;
char *argv[];
{
unsigned div,outport;
int value,baseio,seg320;
int k,l;
char name[20],buffer[24],*malloc();

	/* allocate memory for input and output data */
	for(k=0;k<2*MAXBLK;k++)
	   {
	   storage[k]=(int *)malloc(0x8000);
	   for (l=0;l<0x4000;l++)
	      *(storage[k]+l) = 2048;	/* zero for model 10 data */
	   }
	   lastbuf = MAXBLK-1;

	/* show the sign-on and help screens first */
	/* signon(); */
	showhelp();

	/* setup data pass thru on 320 */
	/* printf("\n\n\t\t\tSetting up data pass thru operation on TMS320\n"); */
	/* data_io(-1); */

	/* display and edit the data area */
	init_video();
	   {
	   edtbuf(  );
	   }
	close_video();

	/* Close all the files and exit */
	fcloseall();
	exit(0);
}

FILE *getfile()
/* this routine get the name of a file from operator */
/* opens the file and reads it into memory.	     */
{
int count,l;
char name[20],buffer[24];
FILE *fp;

	/* open file to be displayed */
	printf("\nData Source File Name ? ");
	scanf("%s",name);
	if( (fp =fopen(name,"rb")) == NULL )
	   {
	    printf("file open error\n");
	    pktc();
	    return (fp);
	    }
	/* get sample rate */
	printf("\nEnter the sample rate data was recorded at:");
	scanf("%f",&srate);
	if ( srate > 0.0 )
	  srate = 1.0/srate;

	/* Read the file */
	for (k=0;(k<MAXBLK)&&(!feof(fp));k++ )
	   {
	   printf("filling block %d",k);
	   count = fread(storage[k],2,0x4000,fp); /* read the file into array */
	   if (count < 0x4000)
	      for (l=count;l<0x4000;l++)
		 *(storage[k]+l) = 2048;   /* zero for model 10 data */
	   printf("   0x%4x data points read\n",count);
	   lastbuf = k;
	   }
	pktc();
	return (fp);
}

void putfile()
/* this routine get the name of a file from operator */
/* opens the file and writes the memory buffers to it. */
{
int count,l;
char name[20],buffer[24];
FILE *fp;

	/* open file to be displayed */
	printf("\nName of data file to be written: ");
	scanf("%s",name);
	if( (fp =fopen(name,"wb")) == NULL )
	   {
	    printf("file open error\n");
	    pktc();
	    return;
	    }

	/* Write the file */
	for (k=0;k<=lastbuf;k++ )
	   {
	   printf("\nwriting block %d",k);
	   fwrite(storage[k],2,0x4000,fp); /* write the buffer to the file */
	   }

	/* close the file */
	fclose(fp);

	pktc();
	return;
}

void edtbuf(void)
{
int data, done, cc, jj ,kk;

   tbase = 4;			/* start with 1/4 buffer per screen */
   dstep = 128; 		/* move forward 128 samples at a time */
   which = 0;			/* start at begining buffer */
   ioff  = 0;			/* start at begining of buffer */
   dsply_chn = BOTH;		/* display two channels */
   done = FALSE;		/* set done flag */
   cc = ' ';			/* clear input char */

   /* draw first screen */
   newscreen( ioff, which, dsply_chn );

   /* let operator do his thing */
   while (  !done )
      {
      if ((cc=ci())==0)    /* wait for operator to press key */
	 {		   /* extended code key pressed */
	 cc = getch();
	 cflush();	   /* clear any excess inputs */
	 switch (cc)
	   {
	   case 59:	  /* function key 1  - gather data */
	    data_io(1);
	    break;
	   case 60:	  /* function key 2  - send data */
	    data_io(0);
	    break;
	   case 61:	  /* function key 3  - load TMS320 program */
	    load_320();
	    break;
	   case 62:	  /* function key 4  - run D320D debugger */
	    run_debug();
	    break;
	   case 63:	  /* function key 5  - run editor */
	    strtedit();
	    break;
	   case 64:	  /* function key 6  - run MAC320 assembler */
	    run_mac320();
	    break;
	   case 65:	  /* function key 7  - process data with 320 */
	    proc_320();
	    break;
	   case 66:	  /* function key 8  - toggle display data */
	    break;
	   case 67:	  /* function key 9  - load new file */
	    _clearscreen( _GCLEARSCREEN );
	    fclose(fp);        /* close current file */
	    fp = getfile();    /* open the new file */
	    ioff = 0;	       /* reset data pointer */
	    break;
	   case 68:	  /* function key 10 - save data in file */
	    _clearscreen( _GCLEARSCREEN );
	    putfile();	       /* write data to the file */
	    break;
	   case 71:	  /* Home	     - time base up */
	    if (tbase >= 2 )
		tbase /= 2;
	    break;
	   case 72:	  /* Up arrow	      - scale up  */
	    if (scale >= 2 )
		scale /= 2;
	    break;
	   case 75:	  /* Left arrow       - forward one dstep */
	    ioff += abs(dstep);
	    if ( ioff >= 0x4000 )
	      {
	      ioff -= 0x4000;
	      which++;
	      if ( which > lastbuf )
		which = 0;
	      }
	    break;
	   case 77:	  /* Right arrow      - Back one dstep */
	    ioff -= abs(dstep);
	    if ( ioff < 0 )
	      {
	      ioff += 0x4000;
	      which--;
	      if ( which < 0 )
		which = lastbuf;
	      }
	    break;
	   case 79:	  /* End	     - time base down */
	    if (tbase < 0x4000 )
		tbase *= 2;
	    break;
	   case 80:	  /* Down arrow       - scale down and display */
	    if (scale <= 16384)
		scale *= 2;
	    break;
	   case 73:	  /* Page Up	     - double dstep */
	    if ( dstep <= 1024 )
		dstep *= 2;
	    break;
	   case 81:	  /* Page Down	     - halve dstep  */
	    if ( dstep >= 2 )
		dstep /= 2;
	    break;
	   default:    /* don't know what he pressed */
	      ;
	   }
	newscreen( ioff, which, dsply_chn );
	}
      else	/* most others do nothing but redraw screen   */
	{
	 switch (cc)
	    {
	    case '?':
	       showhelp();
	       break;
	    case 27:	       /* ESC get you out */
		return;
		break;
	    default:
		;
	    }
	 newscreen( ioff, which, dsply_chn );
	}
      }
}

void data_io(int dir)
/* data input and output  */
/*   dir = -1 for input and output thru 320 nothing happens in here */
/*   dir = 0 for output   */
/*   dir = 1 for input	  */
{
int baseio, k, seg320, value;
unsigned div;
long count;
float trate,osc,rate;

   /* restore graphics to default state */
   close_video();

   /* setup board to output data */
   baseio=0x300;
   i=2;
   osc = 5.0;
   if (i == 2)
      osc = 6.25;
   printf("Clock input = %f Mhz\n",osc);
   osc = osc * 1.0e6;
   cntrlpt = baseio + 7;
   prt0 = baseio + 4;
   prt2 = baseio + 6;
   inp(cntrlpt);   /* halt */

   /* load the 320 file */
     if ( dir == -1 )  /* pass thru mode */
	 spawnlp(P_WAIT,"intel.exe","intel","strthru","d000","300","0",NULL );
     if ( dir == 1 )   /* input data from 320 */
	 spawnlp(P_WAIT,"intel.exe","intel","gather","d000","300","0",NULL );
     if ( dir == 0 )   /* output data to 320 */
	 spawnlp(P_WAIT,"intel.exe","intel","send","d000","300","0",NULL );

   /* setup sample clock on the model 10 */
   clark: do
     {
     printf("Input desired sample rate in Hz ");
     scanf( "%f",&rate);
     } while ( (rate < 80.0 ) || (rate > 40000.1));

   /* count gives you the oscillator "divisor" just as in SIO's */
   count = osc/rate;
   if ( count < 2 ) count += 1;
   /* set timers for desired sampling rate */
   control_timer(count,osc,&trate);
   printf("\nRate = %f ",trate);
   srate = 1.0 / trate;

   /* make sure 320 is stopped */
   inp(cntrlpt);

   /* wait for the operator to say GO */
   printf("\nPress any key to start....");
   ci();

   if ( dir == 1 )
   /* collect data from 320 until all buffers are full */
      {
      init_video();
      drawscrn();
      inp(prt2);	      /* start 320 */
      for (k=0;k<MAXBLK;k++)
	{
	_setcolor(2+k);  /* make trace green, cyan, red, magenta */
	fulbuf(baseio,k);    /* read in a buffer full of data from 320 */
	lastbuf = k;
	}

      /* stop the 320 program */
      inp(cntrlpt);

      /* reset data buffer and offset pointer to 0 */
      ioff = 0;
      which = 0;
      close_video();
      pktc();
      }

   if ( dir == 0 )
   /* output the buffers to the 320 until operator presses key to stop */
      {
      printf("Sorry, not yet working.");
      pktc();
      }

   if ( dir == -1 )
   /* allow data to pass thru the 320 until another function needes it */
   /* assumes that STRTHRU.HEX has been loaded */
     {
      inp(prt2);	      /* start 320 */
      pktc();
     }

   /* restor our video setup */
   init_video();
   return;
}

void control_timer(long cnt, float osc, float *trate)
{
int ctl,cth;

    /* tell board timer info coming */
    outp(cntrlpt ,CONTROL);

    /* count is timing divisor */
    ctl = cnt & 0xff;		  /* lo byte */
    cth = ( cnt >> 8 ) & 0xff;	  /* hi byte */

    /* control timer 0 */
    outp(prt0 , ctl);
    outp(prt0 , cth);

    /* control timer 2 */
    outp(cntrlpt , CONT2);
    outp(prt2 , ctl);
    outp(prt2 , cth);

    /* compute new sampling rate  for output on screen*/
    *trate = osc/(float)cnt;
}

void fulbuf(int baseio, int which)
{
int i,j,porti;

    j=-1;
    porti=baseio+4;
    for(i=0;i<0x4000;i++)
	{
	while(inp(porti)==j)
	    { }
	j=inp(porti);
	*(storage[which]+i)=(inp(baseio+1)<<8)+inp(baseio);
	_setpixel( i/32, (*(storage[which]+i)-2048)*16/scale );
       }
}

void proc_320(void)
/* process data in input buffer with program in 320 */
{
/* NOTE: The program loaded in to the 320 must operate in interrupt */
/*	 mode to begin processing each data point and change the    */
/*	 value in the 8-bit port when the process is done and ready */
/*	 for a new data point.	The process should pick up the	    */
/*	 data from the 16-bit register (pa7) and return the output  */
/*	 to the 16-bit register (pa5).	See the sample program	    */
/*	 dsproc.asm for a model of the necessary communications     */
/*	 procedure.						    */

int ii, j, kk;
int baseio, seg320;
int intrprt, prt4, strtprt, stopprt;

    baseio = 0x300;
    seg320 = 0xd000;
    strtprt = baseio + 6;
    stopprt = baseio + 7;
    prt4    = baseio + 4;
    intrprt = baseio + 2;

    inp(stopprt);	/* be sure 320 is stopped */
    close_video();	/* clear screen */

    inp(strtprt);	/* start the 320 program */

    for ( kk=0; kk<=lastbuf; kk++)
	{
	printf("Processing buffer %d\r",kk);
	for ( ii=0; ii<0x4000; ii++ )
	    {
	    /* get the current value of the 8-bit port */
	    j = inp(prt4);
	    /* put data into 16-bit register */
	    outp(baseio,(*(storage[kk]+ii)&0xff));
	    outp(baseio+1,((*(storage[kk]+ii)>>8)&0xff));
	    /* trip interrupt line to start 320 process */
	    outp(intrprt, 0);
	    /* wait for the process to complete */
	    while(inp(prt4)==j)
		{ /* wait */ }
	    /* get the processed data and place in the channel 2 buffer */
	    *(storage[kk+MAXBLK]+ii) =(inp(baseio+1)<<8)+inp(baseio);
	   }
	}

   inp(stopprt);	/* were done processing turn 320 off */

   init_video();	/* restore the video system */

}

void load_320(void)
{
char name[20];

   /* restore graphics to default state */
   close_video();

   /* request name of file to load  */
   printf("\nLoad TMS32010 File Name ? ");
   scanf("%s",name);

   /* try to load the file */
   if( spawnlp(P_WAIT,"intel.exe","intel",name,"d000","300","0",NULL )!= 0 )
      {
    /*	printf("Error trying to load file - %s\n",name);*/
      pktc();
      }

   /* restor our video setup */
   init_video();
   return;
}

void run_mac320( void )
{
char name[20];
char buffer[24];
char asmfile[30];
char listfile[30];

   /* restore graphics to default state */
   close_video();

   /* setup MAC320.EXE names */
   strcpy( name, "MAC320", 8 );
   strcpy( buffer, name, 20 );
   strcat( buffer, ".exe", 4 );

   /* get file names from operator */
   printf("\nEnter the name of the file to assemble");
   printf("\n     include the usual switches:");
   gets(asmfile);
   printf("\nEnter list file name:");
   gets(listfile);

   /* try to run the assembler */
   if( spawnlp(P_WAIT,buffer,name,asmfile,listfile,NULL )!= 0 )
      {
      printf("Error trying to run Assembler - %s\n",buffer);
      }
   pktc();

   /* restor our video setup */
   init_video();
   return;
}

void run_debug( void )
{
char name[20];
char buffer[24];

   /* restore graphics to default state */
   close_video();

   strcpy( name, "D320D", 8 );
   strcpy( buffer, name, 20 );
   strcat( buffer, ".exe", 4 );

   /* try to run the debugger */
   if( spawnlp(P_WAIT,buffer,name,NULL )!= 0 )
      {
      printf("Error trying to run Debugger - %s\n",buffer);
      pktc();
      }

   /* restor our video setup */
   init_video();
   return;
}

void strtedit( void )
{
char name[20],buffer[24];

   /* restore graphics to default state */
   close_video();

   /* request name of editor  */
   printf("\nEditor File Name ? ");
   scanf("%s",name);
   strcpy( buffer, name, 20 );
   strcat( buffer, ".exe", 4 );

   /* try to run the editor */
   if( spawnlp(P_WAIT,buffer,name,NULL )!= 0 )
      {
      printf("Error trying to run Editor - %s\n",buffer);
      }

   pktc();

   /* restor our video setup */
   init_video();
   return;
}


void showhelp(void)
{
   _clearscreen( _GCLEARSCREEN );
   printf("\t\t\tDSP Scope Commands ");
   printf("\n\tF1          -- gather data ");
   printf("\n\tF2          -- send data ");
   printf("\n\tF3          -- load TMS320 program ");
   printf("\n\tF4          -- run D320D debugger ");
   printf("\n\tF5          -- start text editor ");
   printf("\n\tF6          -- run MAC320 assembler ");
   printf("\n\tF7          -- process data with TMS320 program ");
   printf("\n\tF8          -- toggle displayed data -NIY");
   printf("\n\tF9          -- load buffer from data file ");
   printf("\n\tF10         -- save buffer to data file ");
   printf("\n\t\tUnused function keys cause current screen to be redisplayed");
   printf("\n\tLeft arrow  -- move forward in data ");
   printf("\n\tRight arrow -- move backward in data ");
   printf("\n\tUp arrow    -- increase volts/div ");
   printf("\n\tDown arrow  -- decrease volts/div ");
   printf("\n\tHome        -- increase sec/div ");
   printf("\n\tEnd         -- decrease sec/div ");
   printf("\n\tPgUp        -- increase data step rate ");
   printf("\n\tPgDn        -- decrease data step rate ");
   printf("\n\tEsc         -- exit from program ");
   printf("\n\t?           -- display this screen ");
   printf("\n\n");
   pktc();
}

void signon(void)
{
   printf("\n\t\t\tDSP SCOPE");
   printf("\n\n\tDSPSCOPE is a general purpose interface for working with");
   printf("\n\tthe Delanco-Spry Model 10 Data Acquisition and Signal ");
   printf("\n\tProcessing board.  Standard AMSAT/TAPR setup of memory at");
   printf("\n\t0xd000 and io ports at 0x300 is assumed.");

   printf("\n\tThis version (ver %s) supports the following functions:",version);
   printf("\n\t\to....oscilloscpe type display for viewing data with controls");
   printf("\n\t\t     to change display scales and to pan through stored data.");
   printf("\n\t\to....EGA color, CGA B/W, and 80x87 support is automatic.");
   printf("\n\t\to....load data previously stored with FROM320 for review.");
   printf("\n\t\to....access the Delanco Spry Debugger D320D.EXE. ");
   printf("\n\t\to....access the MAC320 assembler.");
   printf("\n\t\to....access a text editor of the users choice.");
   printf("\n\t\to....collect data directly from the Model 10 (FROM320).");
   printf("\n\t\to....save data for later review.");
   printf("\n\t\to....load and run a TMS320 program to process stored data");
   printf("\n\t\t     and display both input and output signals.");
   printf("\n\tFuture versions will include the following functions:");
   printf("\n\t\to....output data directly to the Model 10 (TO320).");
   printf("\n\t\to....edit data in the buffer. ");
   printf("\n\t\to....compute and display fft of data in display window.");

   printf("\n\t\tCopyright (C) John Conner, WD0FHG 1988.");
   printf("\n\t\tPermission granted for non-commerical use.");
   pktc();

}
