/* dspscope video routines */

#include "dspscope.h"


void newscreen(int ii, int which, int traces )
{
   drawscrn();
   drw_trc1( ii, which );
   _moveto( 0, 0 );
   drw_trc2( ii, which );
}

void drw_trc1(int ii, int which)
{
int buf, jj, kk;
long data;

   _setcolor( 2 );	/* make trace green */
   _moveto( 0, -offset1 );
   for (kk=0;kk<=(0x4000/tbase);kk++)
      {
      jj = kk + ii;
      if( jj < 0x4000 )
	buf = which;
      else
	{
	buf = which + 1;
	if (buf > lastbuf )
	   buf = 0;
	jj  -=	0x4000;
	}
      data = ((*(storage[buf]+jj))-2048);
      _lineto( kk*tbase/32, -((data*scale/4096)+offset1) );
	      /* note data inversion */
      }
}

void drw_trc2(int ii, int which)
{
int buf, jj, kk;
long data;

   _setcolor( 4 );	/* make trace red */
   _moveto( 0, -offset2 );
   for (kk=0;kk<=(0x4000/tbase);kk++)
      {
      jj = kk + ii;
      if( jj < 0x4000 )
	buf = which;
      else
	{
	buf = which + 1;
	if (buf > lastbuf )
	   buf = 0;
	jj  -=	0x4000;
	}
      data = ((*(storage[buf+MAXBLK]+jj))-2048);
      _lineto( kk*tbase/32, -((data*scale/4096)+offset2) );
	      /* note data inversion */
      }
}

void pktc(void)
{
   printf("\n\t\tPress any key to continue...");
   ci();
}

void cflush(void)
/* flush keyboard input */
{
int kbhit(),getch();
   while (kbhit()!=0)
     {
     getch();
     }
}

int ci(void)
/* wait for key press and return the character input */
{
int kbhit(),getch();
   while (kbhit()==0)
     {};
   return (getch());
}

void init_video()
{
   if ( !_setvideomode( _ERESCOLOR ) )
     {
     if ( !_setvideomode( _HRESBW ) )
       {
       printf("High res video not supported.\n");
       exit(0);
       }
     else
       {	    /* CGA  high res b/w */
       maxxres = 512;
       maxyres = 128;
       scale = maxyres; /* start at +/-5 volt fs */
       }
     }
   else
     {		  /* EGA  High res color */
     maxxres = 512;
     maxyres = 256;
     scale = maxyres; /* start at +/-5 volt fs */

     }
   _getvideoconfig( &config );
   _setbkcolor(0l);

   xmin = (config.numxpixels - maxxres)/2;
   ymin = 0;
   xmax = maxxres + xmin;
   ymax = maxyres + ymin;
   xstep = maxxres / (XGRIDS) + 1;
   ystep = maxyres / (2*YGRIDS) + 1;
   offset1 = maxyres/4;    /* reset the offsets for +/- half scale */
   offset2 = -maxyres/4;

   drawscrn();
}

void drawscrn(void)
{
int x,xx,y,yy;
char buff[255];

   _setlogorg( 0, 0 );
   _clearscreen( _GCLEARSCREEN );

   /* Draw an outline box */
   _setcolor( 9 ); /* outline in LIGHTBLUE  */
   _rectangle(_GBORDER,xmin,ymin,xmax,ymax);

   /* set clip region to match box */
   _setcliprgn( xmin,ymin,xmax,ymax );

   /* set logical max and min */
   lxmin = 0;
   lymin = 0;
   lxmax = xmax - xmin;
   lymax = (ymax - ymin)/2;

   /* set logical origin */
   _setlogorg( xmin, lymax - 1 );

   /* draw in x grids */
   for (xx = xstep;xx < lxmax; xx += xstep)
      {
      _moveto( xx, -lymax );
      _lineto( xx, lymax );
      }

   /* draw zero base line */
   _moveto( 0, 0 );
   _lineto( lxmax, 0 );
   _moveto( 0, 0 );

   /* draw in y grids */
   for ( y = 1; y < YGRIDS; y++)
      {
      yy = y * ystep;
      _moveto( 0, yy );
      _lineto( lxmax, yy );
      _moveto( 0, -yy );
      _lineto( lxmax, -yy );
      }

    /* display current values for grid and steps */
    sprintf(buff, "Vert scale = %6.4f volts/div      Horiz scale = %8.6f sec/div",
		   (float)maxyres/scale, srate*xstep*32./tbase );
    _settextcolor(4);
    _settextposition(23,8);
    _outtext(buff);
    sprintf( buff, "buffer= %3d offset= %6d scale= %5u  tbase= %4d  data steps= %4d",
		    which,ioff,scale,tbase,dstep);
    _settextcolor(4);
    _settextposition(24,5);
    _outtext(buff);

    sprintf( buff, "ESC to Exit              DSPSCOPE ver %4s                 ? for help",
		    version);
    _settextcolor(4);
    _settextposition(25,5);
    _outtext(buff);

    /* put pen at origin */
    _moveto( 0, 0 );
}

void close_video()
{
   /* restore graphics to default state */
   _clearscreen( _GCLEARSCREEN );
   _setvideomode( _DEFAULTMODE );
}
