        page 55,132
        title  megax - ibm ega color graphics routines
        name megax
;
;               megax - ibm ega color graphics routines
;
;         for calling by Microsoft C LARGE model only
;
;               Richard C. Allen    1987 August 13
;
;
;   These routines are designed to drive the standard ibm enhanced graphics
;
;   adaptor card and make direct references to the display adaptor registers
;
;   and refresh memory buffer at 0xA0000 ...
;
;
;       The routines assume that the color adaptor has been set into
;
;   the 640 x 350 16-color graphics mode (mode=10h) ...
;

crt_base        equ 0a000h      ; crt base paragraph
x_max           equ 639         ; maximum x location
y_max           equ 349         ; maximum y location
video           equ 10h         ; bios video driver interrupt

status1         equ 03dah       ; status 1 register
attribute       equ 03c0h       ; attribute controller
sequencer       equ 03c4h       ; sequencer
mapmask         equ 2           ; map mask register in sequencer
graphics        equ 03ceh       ; graphics controller port number
setreset        equ 0           ; set/reset register
srenable        equ 1           ; s/r enable register
fselect         equ 3           ; function select register
bitmask         equ 8           ; bit mask register

grafout macro register,absvalue ; absvalue to a graf register
        mov al,register         ; register
        out dx,al               ; set register
        mov al,absvalue         ; load value
        inc dx                  ; -> port
        out dx,al               ; set data
        dec dx                  ; -> controller port
        endm

_text   segment byte public 'CODE'
_text   ends
_data   segment word public 'DATA'
_data   ends
const   segment word public 'CONST'
const   ends
_bss    segment word public 'BSS'
_bss    ends

dgroup  group const, _bss, _data
        assume cs: _text, ds: dgroup, ss: dgroup, es:dgroup

        ; define argument stack offsets

@ab     equ 6           ; stack offset of first argument

arga    equ @ab         ; stack location of first int argument
argb    equ @ab+2       ; stack location of second int argument
argc    equ @ab+4       ; stack location of third int argument
argd    equ @ab+6       ; stack location of fourth int argument
arge    equ @ab+8       ; stack location of fifth int argument
argf    equ @ab+10      ; stack location of sixth int argument
argg    equ @ab+12      ; stack location of seventh int argument

procedure       macro name
        public name
name    proc far
        endm

penter  macro
        push bp
        mov bp,sp
        push es
        push di
        push si
        endm

pleave  macro
        pop si
        pop di
        pop es
        pop bp
        ret
        endm

_text   segment         ; code in this segment

        page

;
;       eg_zor(segment,offset,mask,npix)
;
        procedure _eg_zor

        penter
        push ds

        mov dx,sequencer        ; -> seq
        grafout mapmask,1       ; select data plane only

        mov ax,arga[bp]         ; segment
        mov ds,ax
        mov bx,argb[bp]         ; es:bx -> object byte
        mov dx,argc[bp]         ; dx = mask
        mov cx,argd[bp]         ; cx = count
        mov si,-80              ; si = bytes per row
zor_loop:
        mov al,[bx]             ; fetch data
        or al,dl                ; or in new data
        mov [bx],al             ; restore the data
        add bx,si               ; -> next byte
        loop zor_loop           ; continue

;
; restore registers to default value
;

        mov dx,sequencer        ; -> seq
        grafout mapmask,15      ; select all planes

        pop ds
        pleave

_eg_zor  endp


;
;       eg_zand(segment,offset,mask,npix)
;
        procedure _eg_zand


        penter
        push ds

        mov dx,sequencer        ; -> seq
        grafout mapmask,1       ; select data plane only

        mov ax,arga[bp]         ; segment
        mov ds,ax
        mov bx,argb[bp]         ; es:bx -> object byte
        mov dx,argc[bp]         ; dx = mask
        mov cx,argd[bp]         ; cx = count
        mov si,-80              ; si = bytes per row
zand_loop:
        mov al,[bx]             ; fetch data
        and al,dl               ; and off old data
        mov [bx],al             ; restore the data
        add bx,si               ; -> next byte
        loop zand_loop           ; continue

;
; restore registers to default value
;

        mov dx,sequencer        ; -> seq
        grafout mapmask,15      ; select all planes

        pop ds
        pleave

_eg_zand endp


        page
;
;       eg_read(offset,segment,dest,count)      /* read from memory */
;       int offset;             /* source offset */
;       int segment;            /* source segment */
;       int *dest;              /* -> destination */
;       int count;              /* number of words to read */
;
        procedure _eg_read

        penter
        push ds
        mov si,arga[bp]         ; source offset
        shl si,1                ; tms320 is a 16 bit machine
        mov ax,argb[bp]         ; source segment
        mov ds,ax               ; ds:si -> source
        les di,argc[bp]         ; es:di -> destination
        mov cx,arge[bp]         ; cx is count
        cld                     ; do it forward
        rep movsw               ; move it
        pop ds
        pleave

_eg_read endp

;
;       eg_write(offset,segment,source,count)      /* write to memory */
;       int offset;             /* dest offset */
;       int segment;            /* dest segment */
;       int *dest;              /* -> source */
;       int count;              /* number of words to write */
;
        procedure _eg_write

        penter
        push ds
        mov di,arga[bp]         ; dest offset
        shl di,1                ; tms320 is a 16 bit machine
        mov ax,argb[bp]         ; dest segment
        mov es,ax               ; es:di -> destination
        lds si,argc[bp]         ; ds:si -> source
        mov cx,arge[bp]         ; cx is count
        cld                     ; do it forward
        rep movsw               ; move it
        pop ds
        pleave

_eg_write endp

;
;       eg_swap(p,n)            /* swap bytes in place */
;       int *p;                 /* -> first word */
;       int n;                  /* number of words */
;
        procedure _eg_swap

        penter

        les bx,arga[bp]         ; es:bx -> array
        mov cx,argc[bp]         ; fetch count
swloop:
        mov ax,es:[bx]          ; fetch aword
        xchg ah,al              ; swap bytes
        mov es:[bx],ax          ; restore the word
        add bx,2                ; bump vector
        loop swloop             ; and continue

        pleave

_eg_swap endp


        page
;
;       eg_mode(mode)           /* set crt mode */
;       int mode;               /* mode value (bit 7 on for no clear)
;
        procedure _eg_mode

        penter

        mov ax,arga[bp]         ; fetch mode
        mov ah,0                ; set mode
        int video               ; do it ...
        
        pleave

_eg_mode endp

;
;       eg_bs(k)        /* returns k with bytes swapped */
;       int k;
;
        procedure _eg_bs

        penter

        mov ax,arga[bp]         ; fetch value
        xchg ah,al              ; swap the bytes ...
        
        pleave

_eg_bs   endp


        page
;
;       eg_fill(color)          /* fill screen with color */
;       int color;
;
        procedure _eg_fill
        
        penter

        mov dx,graphics         ; graphics controller port
;
;  select unmodified data mode (as opposed to xor or and etc.. )
;
        grafout fselect,0       ; select normal mode

;
;  output color to the set/reset register
;
        mov al,setreset         ; register
        out dx,al               ; set set/reset register
        mov ax,arga[bp]         ; fetch color
        and al,15               ;  and isolate it
        inc dx                  ; -> port
        out dx,al               ; set color
        dec dx                  ; -> controller port
;
;  enable all four planes
;       
        grafout srenable,15     ; enable all planes

        mov cx,28000            ; 80 * 350  byte count
        push ds                 ; save ds
        mov ax,crt_base
        mov ds,ax               ; -> screen
        mov es,ax               ; -> screen
        mov si,0                ; 0 offset
        mov di,0                ; 0 offset
        grafout bitmask,0ffh    ; all bits
        cld                     ; do it forward
        rep movsb               ; do the dummy read/write ops

        pop ds                  ; restore local data seg
;
; restore registers to default value
;
        grafout setreset,0
        grafout srenable,0
        grafout bitmask,0ffh

        pleave

_eg_fill endp

;
;       eg_deedle(address,count)        /* do graphics deedle */
;       char *address;                  /* start address */
;       int count;                      /* number of iterations */
;
        procedure _eg_deedle

        penter
        push ds
        
        mov cx,argc[bp]         ; fetch count
        test cx,0ffffh          ; check for zero count
        jz eg_deedle_exit       ; leave if zero

        lds si,arga[bp]         ; fetch the pointer
        mov ax,ds
        mov es,ax               ; to both
        mov di,si               ;  ds:si and es:di -> first byte
        cld                     ; do it forward
        rep movsb               ; do the deedle
        
eg_deedle_exit:

        pop ds
        pleave

_eg_deedle endp

        page
;
;       eg_dot(x,y,color)       /* write a dot using the bios */
;       int x,y,color;          /* where and how */
;
        procedure _eg_dot

        penter
        
        mov bh,0                ; page 0
        mov cx,arga[bp]         ; x
        mov dx,argb[bp]         ; y
        mov ax,argc[bp]         ; color
        mov ah,0ch              ; write dot
        int video               ; do it ...
        
        pleave

_eg_dot  endp

;
;       eg_wait()       /* wait for start of vert retrace */
;
        procedure _eg_wait

        push ax
        push dx
        
        mov dx,status1          ; -> status port

eg_wait_1:

        in al,dx                ; fetch status
        test al,8               ; vert set?
        jnz eg_wait_1           ; yes, wait for it to drop

eg_wait_2:

        in al,dx                ; fetch status
        test al,8               ; vert set?
        jz eg_wait_2            ; no, wait for it to rise

        pop dx
        pop ax
        ret

_eg_wait endp

if 0
;
;       eg_palx(color,value)     /* set ega palette thru bios */
;       int color;              /* color 0 to 15 */
;       int value;              /* value for palette 0 0 r'g'b'r g b */
;
        procedure _eg_palx

        penter
        
        mov ax,arga[bp]         ; fetch color code
        mov bl,al               ;  to bl ...
        mov ax,argb[bp]         ; fetch value code
        mov bh,al               ;  to bh ...
        mov ah,10h              ; set palette register
        mov al,0                ; set individual register
        int video               ; invoke bios
        
        pleave

_eg_palx  endp

else

;
;       eg_palx(color,value)     /* set ega palette directly */
;       int color;              /* color 0 to 15 */
;       int value;              /* value for palette 0 0 r'g'b'r g b */
;
;       THIS ROUTINE IS CALLED BY THE USER VERSION eg_pal
;
;       AND DOES NOT SAVE THE PALETTE INFO
;
        procedure _eg_palx

        penter

        call _eg_wait           ; wait for vert retrace

        mov ax,arga[bp]         ; fetch color code
        and al,15               ; isolate color
        mov dx,attribute        ; -> attr chip
        out dx,al               ; set color
        mov ax,argb[bp]         ; fetch value code
        and al,3fh              ; rgbRGB
        out dx,al               ; and output it

        mov al,20h
        out dx,al               ; reenable video
        mov dx,status1
        in al,dx                ; clear adr/data ff by reading status1

        pleave

_eg_palx  endp
endif

        page
;
;       eg_plot(x,y,z)          /* plot a point */
;       int x, y, z;            /* col, row, and color */
;
        procedure _eg_plot

        penter
        
        mov ax,crt_base         ; -> base page
        mov es,ax               ; es: has crt base

        mov si,arga[bp]         ; fetch x
        mov di,argb[bp]         ; fetch y
        mov dx,argc[bp]         ; fetch color
        
        call setpt
        
        pleave

_eg_plot endp

;
;       setpt will plot a point
;       si = x
;       di = y
;       dx = color
;
;   assumes es: has crt base address
;

setpt   proc near

        push bx
        push si
        push ax
        push dx
        push cx
        
        mov bx,dx               ; save color
        mov cx,si               ; save x
        
        mov dx,graphics         ; graphics controller port
;
;  select unmodified data mode (as opposed to xor or and etc.. )
;
        grafout fselect,0       ; select normal mode

;
;  output color to the set/reset register
;
        mov al,setreset         ; register
        out dx,al               ; set set/reset register
        mov ax,bx               ; fetch color
        and al,15               ;  and isolate it
        inc dx                  ; -> port
        out dx,al               ; set color
        dec dx                  ; -> controller port
;
;  enable all four planes
;       
        grafout srenable,15     ; enable all planes

;
;  set a one bit mask according to low 3 x bits
;
        mov al,bitmask          ; -> bitmask register
        out dx,al               ; select it
        inc dx                  ; -> port
        mov si,cx               ; fetch x
        and si,7                ; isolate bit number
        mov al,bmask[si]        ; fetch mask
        out dx,al               ; set bit mask
        dec dx                  ; -> controller port
;
;  calculate effective address for the byte containing the dot
;
        mov si,di               ; fetch y
        sal si,1                ; *2
        mov bx,rtab[si]         ; bx = y * 80
        
        mov ax,cx               ; fetch x
        sar ax,1                ; / 2
        sar ax,1                ; / 4
        sar ax,1                ; / 8
        add bx,ax               ; bx has byte offset
        inc byte ptr es:[bx]    ; dummy read/write
;
; restore registers to default value
;
        grafout setreset,0
        grafout srenable,0
        grafout bitmask,0ffh
        
        pop cx
        pop dx
        pop ax
        pop si
        pop bx
        ret

setpt   endp

;
;       setdot will plot a point at current color (used by line)
;       si = x
;       di = y
;
;   assumes es: has crt base address
;           caller does pre- and post-housekeeping
;

setdot  proc near

        push bx
        push si
        push ax
        push dx
        push cx
        
        mov cx,si               ; save x
        
        mov dx,graphics         ; graphics controller port
;
;  set a one bit mask according to low 3 x bits
;
        mov al,bitmask          ; -> bitmask register
        out dx,al               ; select it
        inc dx                  ; -> port
        mov si,cx               ; fetch x
        and si,7                ; isolate bit number
        mov al,bmask[si]        ; fetch mask
        out dx,al               ; set bit mask
        dec dx                  ; -> controller port
;
;  calculate effective address for the byte containing the dot
;
        mov si,di               ; fetch y
        sal si,1                ; *2
        mov bx,rtab[si]         ; bx = y * 80
        
        mov ax,cx               ; fetch x
        sar ax,1                ; / 2
        sar ax,1                ; / 4
        sar ax,1                ; / 8
        add bx,ax               ; bx has byte offset
;        mov al,es:[bx]          ; dummy read
;        mov es:[bx],al          ; dummy write
        inc byte ptr es:[bx]    ; dummy read/write
        
        pop cx
        pop dx
        pop ax
        pop si
        pop bx
        ret

setdot   endp
        
        page
;
;       eg_line(x1,y1,x2,y2,z)  /* plot a line */
;       int x1, y1, x2, y2;     /* endpoints */
;       int z;                  /* color */
;

lx1     equ arga[bp]
ly1     equ argb[bp]
lx2     equ argc[bp]
ly2     equ argd[bp]
lz      equ arge[bp]

        procedure _eg_line

        penter

        mov ax,crt_base         ; -> crt
        mov es,ax               ; -> the crt base

        mov dx,graphics         ; graphics controller port
;
;  select unmodified data mode (as opposed to xor or and etc.. )
;
        grafout fselect,0       ; select normal mode

;
;  output color to the set/reset register
;
        mov al,setreset         ; register
        out dx,al               ; set set/reset register
        mov ax,lz               ; fetch color
        and al,15               ;  and isolate it
        inc dx                  ; -> port
        out dx,al               ; set color
        dec dx                  ; -> controller port
;
;  enable all four planes
;       
        grafout srenable,15     ; enable all planes

;
; set up x and y updates
;
        mov si,1                ; start with positive 1 for x update
        mov di,1                ; start with positive 1 for y update
;
; find |y2 - y1|
;
        mov dx,ly2              ; get y2
        sub dx,ly1              ; subtract y1
        jge storey              ; skip if y2-y1 is non-negative
        neg di                  ; move in negative y direction
        neg dx                  ; abs(y2-y1)
storey:
        mov deldy,di            ;store y update for diagonal moves
        
;
; find abs(x2-x1)
;
        mov cx,lx2              ; get x2
        sub cx,lx1              ; - x1
        jge storex              ; skip if x2-x1 is non-negative
        neg si                  ; negative x direction
        neg cx                  ; abs(x2-x1)
storex:
        mov deldx,si            ; save x update for diagonals
;
; sort abs(y2-y1) and abs(x2-x1)
;
        cmp cx,dx               ; compare dels delp
        jge setdiag             ; skip if straight moves in x direction
        mov si,0                ; straight vertical, kill x update
        xchg cx,dx              ; swap deltas
        jmp storedelsxy

setdiag:
        mov di,0                ; kill y update
;
; store dels, delp, delsx, and delsy
;
storedelsxy:
        mov dels,cx             ; delta straight
        mov delp,dx             ; delta in perpendicular to straight
        mov delsx,si            ; delta x straight
        mov delsy,di            ; delta y straight
;
; get initial values for x and y
;
        mov si,lx1
        mov di,ly1
;
; compute initial value and increments for error function
;
        mov ax,delp
        sal ax,1                ; 2*delp
        mov delse,ax            ; delta for staright moves

        sub ax,cx               ; 2*delp - dels
        mov bx,ax               ; initial value

        sub ax,cx               ; 2*delp - 2*dels
        mov delde,ax            ; delta if diagonal
;
; adjust count
;
        inc cx

;
; main loop structure
;

lineloop:
        call setdot             ; plot the point
        cmp bx,0                ; straight or diagonal?
        jge diagonal
;
; case for straight move
;

straight:
        add si,delsx            ; update x
        add di,delsy            ; update y
        add bx,delse            ; update error term
        loop lineloop           ; and continue
        jmp lineexit            ; all done
;
; case for diagonal move
;

diagonal:
        add si,deldx            ; update x
        add di,deldy            ; update y
        add bx,delde            ; update error term
        loop lineloop           ;  and continue ...
;
; exit
;

lineexit:
        
        mov dx,graphics         ; -> graphics controller
;
; restore registers to default value
;
        grafout setreset,0
        grafout srenable,0
        grafout bitmask,0ffh

        pleave

_eg_line endp
        page
;
;       eg_fcp(pch,x,y,z)       /* plot 5x9 character */
;       char *pch;              /* -> top of character table */
;       int x,y,z;              /* where and how ... */
;
        procedure _eg_fcp

        penter

        mov ax,crt_base         ; -> crt
        mov es,ax               ; -> the crt base

        mov dx,graphics         ; graphics controller port
;
;  select unmodified data mode (as opposed to xor or and etc.. )
;
        grafout fselect,0       ; select normal mode

;
;  output color to the set/reset register
;
        mov al,setreset         ; register
        out dx,al               ; set set/reset register
        mov ax,arge[bp]         ; fetch color
        and al,15               ;  and isolate it
        inc dx                  ; -> port
        out dx,al               ; set color
        dec dx                  ; -> controller port
;
;  enable all four planes
;       
        grafout srenable,15     ; enable all planes

        mov bx,arga[bp]         ; destination offset
        mov di,argd[bp]         ; fetch y
        mov cx,9                ; number of rows to plot

fcp_loop:
        push ds                 ; save ds
        mov ax,argb[bp]         ; fetch table segment
        mov ds,ax               ; set segment
        mov al,[bx]             ; fetch row data
        inc bx                  ; -> next row
        pop ds
        mov si,argc[bp]         ; fetch x

fcp_row:
        test al,0ffh            ; row zero?
        jz fcp_next             ; yes, continue
        test al,80h             ; high bit set?
        jz fcp_row1             ; no
        call setdot             ; yes, plot the point
fcp_row1:
        inc si                  ; x++
        sal al,1                ; shift the mask
        jmp fcp_row             ; continue row

fcp_next:
        inc di                  ; y++
        loop fcp_loop           ; do next row

        mov dx,graphics         ; -> graphics controller
;
; restore registers to default value
;
        grafout setreset,0
        grafout srenable,0
        grafout bitmask,0ffh

        pleave

_eg_fcp  endp

        page
;
;       eg_byte_plot(d0,d1,d2,d3,screen)
;       int d0, d1, d2, d3;                 /* the plane bytes */
;       char *screen;                       /* -> effective byte */
;
        procedure _eg_byte_plot

        penter

        mov ax,crt_base         ; -> crt
        mov es,ax               ; es:bx -> the effective byte

        les bx,arge[bp]         ; fetch screen pointer
        
        mov dx,graphics         ; graphics controller port

;
;  select unmodified data mode (as opposed to xor or and etc.. )
;
        grafout fselect,0       ; select normal mode

;
;  enable all four planes
;       
        grafout srenable,15     ; enable all planes

;
;   plot loop
;
;       es:bx -> effective byte
;       si       has iteration count 8 <
;       cl       mask bit
;       ch       color
;
        mov si,8                ; bit count
        mov cl,10000000b        ; initial mask

plot_loop:

        sub ch,ch               ; zero color
;
;   translate the four input bytes into color
;
        mov ax,arga[bp]         ; fetch d0
        and al,cl               ; bit set?
        jz pl0                  ; no
        or ch,1                 ; yes, set it
pl0:

        mov ax,argb[bp]         ; fetch d1
        and al,cl               ; bit set?
        jz pl1                  ; no
        or ch,2                 ; yes, set it
pl1:

        mov ax,argc[bp]         ; fetch d2
        and al,cl               ; bit set?
        jz pl2                  ; no
        or ch,4                 ; yes, set it
pl2:

        mov ax,argd[bp]         ; fetch d3
        and al,cl               ; bit set?
        jz pl3                  ; no
        or ch,8                 ; yes, set it
pl3:

;
;       ch has color
;
;
;  output color to the set/reset register
;
        mov al,setreset         ; register
        out dx,al               ; set set/reset register
        mov al,ch               ; fetch color
        inc dx                  ; -> port
        out dx,al               ; set color
        dec dx                  ; -> controller port
;
;  set a one bit mask according to low 3 x bits from cl
;
        mov al,bitmask          ; -> bitmask register
        out dx,al               ; select it
        inc dx                  ; -> port
        mov al,cl               ; fetch mask
        out dx,al               ; set bit mask
        dec dx                  ; -> controller port

        shr cl,1                ; shift mask

        inc byte ptr es:[bx]    ; dummy read/write to plot

        dec si                  ; done?
        jnz plot_loop           ; no, continue

;
; restore registers to default value
;
        grafout setreset,0
        grafout srenable,0
        grafout bitmask,0ffh

        pleave

_eg_byte_plot endp

        page

_text   ends

_data   segment                 ; data here
        
deldy   dw ?
dels    dw ?
delp    dw ?
delsx   dw ?
delsy   dw ?
delse   dw ?
delde   dw ?
deldx   dw ?

bmask   db 80h,40h,20h,10h,8,4,2,1      ; bit masks

;
; rtab[n] = n * 80   for fast multiply by 80
;

rtab dw     0,   80,  160,  240,  320,  400,  480,  560,  640,  720,  800
     dw   880,  960, 1040, 1120, 1200, 1280, 1360, 1440, 1520, 1600, 1680
     dw  1760, 1840, 1920, 2000, 2080, 2160, 2240, 2320, 2400, 2480, 2560
     dw  2640, 2720, 2800, 2880, 2960, 3040, 3120, 3200, 3280, 3360, 3440
     dw  3520, 3600, 3680, 3760, 3840, 3920, 4000, 4080, 4160, 4240, 4320
     dw  4400, 4480, 4560, 4640, 4720, 4800, 4880, 4960, 5040, 5120, 5200
     dw  5280, 5360, 5440, 5520, 5600, 5680, 5760, 5840, 5920, 6000, 6080
     dw  6160, 6240, 6320, 6400, 6480, 6560, 6640, 6720, 6800, 6880, 6960
     dw  7040, 7120, 7200, 7280, 7360, 7440, 7520, 7600, 7680, 7760, 7840
     dw  7920, 8000, 8080, 8160, 8240, 8320, 8400, 8480, 8560, 8640, 8720
     dw  8800, 8880, 8960, 9040, 9120, 9200, 9280, 9360, 9440, 9520, 9600
     dw  9680, 9760, 9840, 9920,10000,10080,10160,10240,10320,10400,10480
     dw 10560,10640,10720,10800,10880,10960,11040,11120,11200,11280,11360
     dw 11440,11520,11600,11680,11760,11840,11920,12000,12080,12160,12240
     dw 12320,12400,12480,12560,12640,12720,12800,12880,12960,13040,13120
     dw 13200,13280,13360,13440,13520,13600,13680,13760,13840,13920,14000
     dw 14080,14160,14240,14320,14400,14480,14560,14640,14720,14800,14880
     dw 14960,15040,15120,15200,15280,15360,15440,15520,15600,15680,15760
     dw 15840,15920,16000,16080,16160,16240,16320,16400,16480,16560,16640
     dw 16720,16800,16880,16960,17040,17120,17200,17280,17360,17440,17520
     dw 17600,17680,17760,17840,17920,18000,18080,18160,18240,18320,18400
     dw 18480,18560,18640,18720,18800,18880,18960,19040,19120,19200,19280
     dw 19360,19440,19520,19600,19680,19760,19840,19920,20000,20080,20160
     dw 20240,20320,20400,20480,20560,20640,20720,20800,20880,20960,21040
     dw 21120,21200,21280,21360,21440,21520,21600,21680,21760,21840,21920
     dw 22000,22080,22160,22240,22320,22400,22480,22560,22640,22720,22800
     dw 22880,22960,23040,23120,23200,23280,23360,23440,23520,23600,23680
     dw 23760,23840,23920,24000,24080,24160,24240,24320,24400,24480,24560
     dw 24640,24720,24800,24880,24960,25040,25120,25200,25280,25360,25440
     dw 25520,25600,25680,25760,25840,25920,26000,26080,26160,26240,26320
     dw 26400,26480,26560,26640,26720,26800,26880,26960,27040,27120,27200
     dw 27280,27360,27440,27520,27600,27680,27760,27840,27920

_data   ends

	end

