;
;        osc - test oscillators on tms32010
;
;            R. C. Allen, 8 August 1987
;
;   sin table (stab.hex) must be loaded previously
;

dac:    equ 4           ; dac port number
pcio:   equ 7           ; pc reads/writes to this port

sintab: equ 700h        ; top of memory is sin table
                        ; 700h to EFFh  (F00 up for breakpoints)

        org 0
        b start         ; do not change this
        b intr          ; interrupts here
;
; define data memory (constants in program memory moved to registers)
;

rx:     equ 0           ; temp reg      these overlay
ry:     equ 1           ; temp reg      the three words
rz:     equ 2           ; temp reg      at org 0
temp:   equ 3           ; temp data location
value:  equ 4           ; running value

;
;    these locations are initilisation values in code memory
;    AND registers in data memory which are set to the code
;    memory values at startup
;

        org 5

k0:     dw 0            ; zero constant
k1:     dw 1            ; one constant
km1:    dw -1           ; all ones constant
hff:    dw 0ffh         ; 0xff
h800:   dw 800h         ; 0x800 (dac offset)
phase:  dw 0            ; phase register
asint:  dw sintab       ; address of sintable
delta:  dw 600          ; delta phase
vect:   dw 0            ; table vector
amask:  dw 7ffh         ; table address mask
scale:  dw 7fffh        ; output scale (q15 7fff = .99999)
h7fff:  dw 7fffh        ; 0x7fff
rrate:  dw 10           ; ramp rate
nused:  equ $-k0        ; number of registers to initialise


;
; interrupt processor
;

intr:   ret             ; do nothing

;
; startup - initialise data memory from program memory constants
;

start:  ldpk 0          ; -> low data page
        lack 1          ; fetch a 1
        sacl rz         ; rz = 1
        lack nused      ; number of registers used
        sacl rx         ; set counter
        lack k0         ; first reg to initialise
        sacl ry         ; save it
        larp 0          ; use ar0
        lark 0,k0       ; first reg
inloop:   
        lac ry          ; program memory location
        tblr *+         ; read from pgm mem to data mem
        lack 1          ; fetch a one
        add ry          ; bump ry
        sacl ry         ; ry++
        lac rx          ; iteration counter
        sub rz          ; -1
        sacl rx         ; restore it
        bgz inloop      ; continue 'til done
;
; output oscillator
;

loop:
;        in delta,pcio   ; read the delta phase from host
;        in scale,pcio   ; read the scale from host
        lac phase,11    ; fetch top 11 of phase to ach
        sach vect       ; copy to vect
        lac vect        ; low 11 has offset
        and amask       ; isolate offset
        add asint       ; -> start of sin table
        tblr temp       ; read the sample value
        lt temp         ; fetch sample for scaling
        mpy scale       ; scale the sample
        pac             ; result to ac
        sach temp,1     ; save it scaled up one place
        lac temp        ; restore lo
        xor h800        ; convert to offset for dac
        sacl temp       ; restore it
        out temp,dac    ; output the sample
        lac phase       ; fetch phase
        add delta       ; add delta phase
        sacl phase      ; restore the phase

        lac scale       ; fetch scale
        sub rrate       ; bump it
        and h7fff       ; isolate
        sacl scale      ;  and restore

        lack 20         ; delay 
        call delay
        b loop          ; continue


;
; delay subroutine - ac has delay value > 0
;
;       1 usec per count (20.0 MHz clock)
;

delay:  sub k1          ; decrement   .2
        nop             ;             .2
        nop             ;             .2
        bgz delay       ; 'til done   .4
        ret

        end

