/*
        rcaspect - cleaned up version of newspect.c

        R. C. Allen, 11 August 1987

*/

#include    <stdio.h>
#include    <dos.h>
#include    <memory.h>
#include    <process.h>

    /* define 8254 control bytes */

#define CONTROL 0x34        /* timer one setup byte */
#define CONT2 0xb4          /* timer 2 setup byte */

    /* define pc keyboard specials */

#define COMLINE 62          /*  (f4)  */
#define UP 72
#define DOWN 80
#define PGUP 73
#define PGDOWN 81
#define RIGHT 77
#define LEFT 75  
#define CNRIGHT 116
#define CNLEFT 115  
#define HOME 71
#define END 79
#define CNHOME 119
#define CNEND 117
#define CLOSE 61            /* (f3   */
#define TCONSTU 60          /* (f2)  */ 
#define TCONSTD 59          /* (f1)  */ 
#define TCONST_NEW 68       /* (f10) */

unsigned int seg320;    /* memory segment (para) for tms320 board */
unsigned int io320;     /* base i/o address for tms320 board */
unsigned int cntrlpt;   /* timer control port */
unsigned int prt0;      /* timer 0 port */
unsigned int prt1;      /* timer 1 port */
unsigned int prt2;      /* timer 2 port */

int spect_count;
int y[1024];
int z[1024];
float norm[256];
float trate;

main(argc,argv)
int argc;
char *argv[];
{
    char c,scr_csts();
    int cmin,j,jh,k,i,scale,offset,kount;
    int ct,binmax;
    long count,jjj,jj,jjmax,jjmore,jjcnt;
    float zhold[513],zsum[256],tconst,rate,osc,amplitude;

    seg320 = 0xd000;        /* set 320 board mem paragraph */
    tconst = 0.5;
    jjcnt = 0;
    io320 = 0x300;          /*  base i/o address for 320 board */
    osc = 25.0e6/4.0;       /*  basic board oscillator */

    cntrlpt = io320 + 7;    /* define 8254 timer ports */
    prt0 = io320 + 4;
    prt2 = io320 + 6;

    scale = 5;
    jjmore = 0;
    offset = 180;

clark:

    do          /* fetch input sample rate */
    {
        cls();
        printf("Input desired sample rate in Hz ");
        scanf( "%f",&rate);
    }
    while ( (rate < 80.0 ) || (rate > 40000.1));

/* count gives you the oscillator "divisor" just as in SIO's */

    count = osc/rate;
    cmin = osc / 40000.0;
    if (count < 2) count = 2;
    trate = osc / ((float) count);
    printf("\nRate = %f ",trate);

    /* set timers for desired sampling rate */

    control_timer(count,osc,&trate);

    /* sets screen in high res mode using ansi.sys */
    /* cursor positioned by call to DOS, VERY clone perfect dependent */

    scr_rowcol(24,1);
    printf("%7.2f Hz     ",trate);

           /* time delay for 8254 & clear holding arrays */

    for (k = 0; k < 20000; k++)
    {
        if (k < 256) norm[k] = 0.0;          
        if (k < 513) zhold[k] = 0.0;
    }     

    inp(io320 + 7);   /* halt 320 */

           /* Load the fft and square routines */
    spawnl(P_WAIT,"intel",argv[0],"ffthy","d000","300","0",NULL);

    /* Run it once to set up arrays and have proper values in the ports */

    inp(io320 + 6);                     /* go 320 */
    while(inp(io320) != 88);            /* wait for done */
    inp(io320 + 7);                     /* stop 320 */

         /* Generate Reference Baseline Spectrum */

    printf("\n How many samples in reference spectrum? ");
    scanf("%d",&spect_count);
    ref_spect(spect_count);
    printf("\nReference spectrum --- Hit any character when ready");
    ci(); cls(); scr_rowcol(1,1);
    printf("\n How many samples for each displayed spectrum? ");
    scanf("%d",&spect_count);

/* Set it up to run forever until ^C or F3 to change screen and quit*/

    hres(); kount=0; 
    while (1) 
    {
        if ((c = scr_csts()) != 0 )     /* c has key code */
        {
            switch(c)
            {
            case COMLINE:
                co80();
                goto clark;         /* barf */
                break;

            case UP:
                scale -= 1;
                if (scale < 0) scale = 0;
                break;

            case DOWN:
                scale += 1;
                if (scale > 15) scale = 15;
                break;

            case PGUP:
                offset -= 10;
                if (offset < 0) offset = 0;
                break;

            case PGDOWN:
                offset += 10;
                if (offset > 180) offset = 180;
                break;

            case CLOSE:
                inp(io320 + 7); /* halt */
                co80();
                exit();
                break;

            case HOME:
                jjmore -= 1;
                if (jjmore < 0) jjmore = 0;
                break;

            case END:
                jjmore += 1;
                if (jjmore > 10000) jjmore = 10000;
                break;

            case CNHOME:
                jjmore /= 2;
                if (jjmore < 0) jjmore = 0;
                break;

            case CNEND:
                jjmore *= 2;
                if (jjmore > 10000) jjmore = 10000;
                break;

            case TCONSTU:
                tconst *= 2.0;
                if (tconst>1.0) tconst=1.0;
                break;

            case TCONST_NEW:
                cls(); printf("%3d\n",c);
                printf(" tconst currently = %9.7f --- Enter new value: ",tconst);
                scanf("%f",&tconst); cls();
                if (tconst >  1.0) tconst = 1.0;
                if (tconst < 0.0001) tconst = 0.0001;     
                break;

                 case TCONSTD:
                tconst *= 0.5;
                if (tconst < 0.0001) tconst = 0.0001;     
                break;

            default:
                break;
            }               /* end switch(c) */
        }                   /* end if */

        if (z[100]+z[110]+z[120]+z[130]+z[140]+z[150] > 3)  /* is there data? */
        {          /* bypass the following loop if there isn't */
            for (k = 0; k < 256; k++)
            {
                 if (kount == 0) zsum[k] = (float) z[k];
                 if (kount != 0) zsum[k] = zsum[k] + (float) z[k];
             }    
            kount++;
            if (kount >= spect_count)
            {   
                 kount=0;
                for (k = 0; k < 256; k++)
                {
                    zsum[k] = zsum[k] * norm[k] * 10.0;        
                }
                amplitude=0.;
                binmax=128;
                for (k = 0; k < 512; k += 2)
                {
                    zhold[k] = (1.0 - tconst) * zhold[k] + zsum[k / 2] * tconst;
                    y[k]= offset - ((int) zhold[k] >> scale);
                    zsum[k / 2] = 0;
                    if (y[k] < 1) y[k] = 1;              /* don't go off the top */
                    if (k > 80 && zhold[k] > amplitude)  /* avoid DC peak */
                    {
                        binmax = k;
                        amplitude = zhold[k];
                    }
                }
                for (k = 1; k < 511; k += 2)
                {
                    y[k] = (y[k - 1] + y[k + 1]) / 2.0;
                }
                 scr_rowcol(24,1);
                printf("Rate= %5.0f Hz Scale= %d tconst= % 5.4f sampl= %d freq=% 5.0f amp=% 4.0f  ",
                    trate,scale,tconst,spect_count,((float)binmax)*trate/1024.0,amplitude);
                sigpts(&y[0],512);      /* show old fft */

            }
        }     /* end of bypass loop -- display counter even if no data at input */
          
        scr_rowcol(1,1);
        printf("% 5d   ",jjcnt++);
        if (jjcnt > 32675) jjcnt = 0;
        fft();
    }
}

fft()       /* wait for fft done, fetch result and start another */
{    
    while (inp(io320) != 88);       /* wait for 320 done */
    inp(io320 + 7);                 /* halt */
    peekwf(0x200,seg320,&z[0],256); /* pull fft result out */
    inp(io320 + 6);                 /* restart */
}

co80()
{
    printf("\x1B[=3h");
}

cls()
{
    printf("\x1B[2J");
}

hres()
{
    printf("\x1b[=6h");
}

scr_rowcol(x,y)
unsigned x,y;
{
    union REGS inregs,outregs;

    inregs.x.ax = 0x200;
    inregs.x.bx = 15;
    inregs.x.dx = (x << 8) + y;
    int86(0x10,&inregs,&outregs);
}

ci()
{
    int kbhit(), getch(), j;

    while (kbhit() == 0);       /* wait for a key */
      j = getch();                /* dump the key */
}

char scr_csts()     /* read a special key */
{
    char c;

    if (kbhit() == 0) return (0);       /* no keys ready */
    if (getch() != 0) return (0);       /* not a special ??? */
    return (getch());
}

control_timer(cnt,osc,trate)
long cnt;
float *trate, osc;
{
    int ctl,cth;

        /* tell board timer info coming */

    outp(cntrlpt ,CONTROL);

        /* count is timing divisor */

    ctl = cnt & 0xff;            /* lo byte */
    cth = (cnt >> 8) & 0xff;    /* hi byte */

        /* control timer 0 */

    outp(prt0, ctl);
    outp(prt0, cth);

        /* control timer 2 */

    outp(cntrlpt, CONT2);
    outp(prt2, ctl);
    outp(prt2, cth);

    /* compute new sampling rate  for output on screen*/

    *trate = osc / (float) cnt;
}

ref_spect(spect_count)
int spect_count;
{
    float t1,t2,t3,t4,norm_max;
    int binmax,jj,k;

    inp(io320 + 6);                             /* go 320 */
    for (jj = 0; jj < spect_count; jj++)        /* for each ref spectrum */
    {
        scr_rowcol(4,1); printf("% 5d  ",jj);
        fft();
        for (k = 0; k < 256; k++)
        {
            norm[k] = norm[k] + (float) z[k];          
        }
    }
    binmax = 128;
    norm_max = -999.0;
    t1 = t2 = t3 = t4 = 0.0;

         /* start with 7-point running average smoothing */

    for (k = 0; k < 249; k++)
    {
        t4 = t3;
        t3 = t2;
        t2 = t1;
        t1 = norm[k] + norm[k + 1] + norm[k + 2] +
             norm[k + 3] + norm[k + 4] + norm[k + 5] + norm[k + 6];
        norm[k] = t4 / 7.0;          /* keep centered at proper index */
        
           if (norm[k] > norm_max)
        {
            binmax = k;
            norm_max = norm[k];
        }    
    }
    for (k = 0; k < 6; k++)
    {
        norm[k] = 0.0;
        norm[256 - k] = 0.0;
    }  
    for (k = 0; k < 512; k += 2)
    {
        y[k] = 180 - (norm[k / 2] / norm_max) * 100.0;
        if (norm[k / 2] > norm_max / 50. && k > 60)
        {
            norm[k / 2] = norm_max / norm[k / 2];    
        }
        else norm[k / 2] = 0.0;
    }    
    for (k = 1; k < 511; k += 2)
    {
        y[k] = (y[k - 1] + y[k + 1]) / 2.0;
    }
    hres(); sigpts(&y[0],512);
       scr_rowcol(24,1);
    printf(
        "Rate = %7.2f Hz                          freq=% 7.1f amp=% 8.1f  ",
        trate, ((float)binmax) * trate / 512.0, norm_max );
}

